@@ -17,45 +17,70 @@
 %% If not, see <http://www.gnu.org/licenses/>.
 
 %% -*- texinfo -*-
+%% @documentencoding UTF-8
 %% @deftypefn  {Function File} {@var{g} =} function_handle (@var{f})
 %% @deftypefnx {Function File} {@var{g} =} function_handle (@var{f1}, @dots{}, @var{fn})
 %% @deftypefnx {Function File} {@var{g} =} function_handle (@dots{}, @var{param}, @var{value})
 %% @deftypefnx {Function File} {@var{g} =} function_handle (@dots{}, 'vars', [@var{x} @dots{} @var{z}])
-%% @deftypefnx {Function File} {@dots{}} function_handle (@dots{}, 'file', @var{name})
+%% @deftypefnx {Function File} {@dots{}} function_handle (@dots{}, 'file', @var{filename})
 %% @deftypefnx {Function File} {@dots{}} function_handle (@dots{}, 'outputs', [@var{o1} @dots{} @var{on}])
 %% Convert symbolic expression into a standard function.
 %%
 %% This can make anonymous functions from symbolic expressions:
 %% @example
-%% syms x y
-%% f = x^2 + sin(y)
-%% h = function_handle(f)
-%% % output: h = @@(x,y) x.^2 + sin(y)
+%% @group
+%% >> syms x y
+%% >> f = x^2 + sin(y)
+%%    @result{} f = (sym)
+%%         2
+%%        x  + sin(y)
+%% >> h = function_handle(f)
+%%    @result{} h =
+%%      @@(x, y) x .^ 2 + sin (y)
+%% @end group
 %% @end example
+%%
+%% Multiple arguments correspond to multiple outputs of the
+%% function.  For example, the final @code{x} in this example
+%% specifies the third output (rather than the input):
+%% @example
+%% @group
+%% >> h = function_handle(x^2, 5*x, x);
+%% >> [a, b, c] = h(2)
+%%    @result{} a = 4
+%%    @result{} b = 10
+%%    @result{} c = 2
+%% @end group
+%% @end example
+%%
 %% The order and number of inputs can be specified:
 %% @example
-%% syms x y
-%% f = x^2 + sin(y)
-%% h = function_handle(f, 'vars', [z y x])
-%% % output: h = @@(z,y,x) x.^2 + sin(y)
+%% @group
+%% >> syms x y z
+%% >> h = function_handle(f, 'vars', [z y x])
+%%    @result{} h =
+%%      @@(z, y, x) x .^ 2 + sin (y)
+%% @end group
 %% @end example
 %%
 %% For compatibility with the Symbolic Math Toolbox in Matlab, we
 %% provide a synonym: @xref{matlabFunction}
 %%
-%% OctSymPy can also generate a @code{.m} file from symbolic
-%% expression:
+%% OctSymPy can also generate an @code{.m} file from a symbolic
+%% expression by passing the keyword @code{file} with a string
+%% argument for @var{filename}.  A handle to the function in the
+%% file will be returned.
+%% Passing an empty @var{filename} creates an anonymous function:
 %% @example
-%% h = function_handle(f, 'file', 'myfcn')
-%% % creates a file called myfcn.m and returns a handle
-%% @end example
-%% Passing an empty filename creates an anonymous function:
-%% @example
-%% h = function_handle(f, 'vars', [z y x], 'file', '')
+%% @group
+%% >> h = function_handle(f, 'file', '')
+%%    @result{} h =
+%%      @@(x, y) x .^ 2 + sin (y)
+%% @end group
 %% @end example
 %%
 %% FIXME: naming outputs with @var{PARAM} as
-%% 'outputs' not implemented.
+%% @code{outputs} not implemented.
 %%
 %% FIXME: does not ``optimize'' code, for example, using common
 %% subexpression elimination.
@@ -100,10 +125,10 @@ function f = function_handle(varargin)
 
     if (~worked)
       if (strcmp(out, 'Language ''octave'' is not supported.'))
-	error('function_handle: your SymPy has no octave codegen, cannot workaround');
+        error('function_handle: your SymPy has no octave codegen, cannot workaround');
       else
-	ou
-	error('function_handle: Some other error from SymPy code gen?  file a bug!');
+        out
+        error('function_handle: Some other error from SymPy code gen?  file a bug!');
       end
     end
     M.name = out{1}{1};
@@ -132,13 +157,10 @@ function f = function_handle(varargin)
               '    return (False, "expected symbols-to-declare to be empty")' ...
               'return (True, s)' };
       [worked, codestr] = python_cmd (cmd, expr);
-      %worked = false;
-      if (worked)
-        codestr = vectorize(codestr);
-      else
+      if (~worked)
         %% SymPy 0.7.5 has no octave_code command
         % Use a crude workaround (e.g., Abs, ceiling will fail).
-        if (str2num(strrep(python_cmd ('return sp.__version__,'),'.',''))<=75 ...
+        if ((python_cmd ('return Version(spver) <= Version("0.7.5"),')) ...
             && strcmp(codestr, 'global name ''octave_code'' is not defined'))
           warning('OctSymPy:function_handle:nocodegen', ...
                   'function_handle: your SymPy has no octave codegen: partial workaround');
@@ -174,66 +196,53 @@ end
 
 %!test
 %! % basic test
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(2*x);
-%! warning(s)
 %! assert(isa(h, 'function_handle'))
 %! assert(h(3)==6)
 
 %!test
 %! % autodetect inputs
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(2*x*y, x+y);
-%! warning(s)
 %! [t1, t2] = h(3,5);
 %! assert(t1 == 30 && t2 == 8)
 
 %!test
 %! % specified inputs
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(2*x*y, 'vars', [x y]);
 %! assert(h(3,5)==30)
 %! h = function_handle(2*x*y, x+y, 'vars', [x y]);
-%! warning(s)
 %! [t1, t2] = h(3,5);
 %! assert(t1 == 30 && t2 == 8)
 
 %!test
 %! % cell arrays for vars list
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(2*x*y, x+y, 'vars', {x y});
 %! [t1, t2] = h(3,5);
 %! assert(t1 == 30 && t2 == 8)
 %! h = function_handle(2*x*y, x+y, 'vars', {'x' 'y'});
-%! warning(s)
 %! [t1, t2] = h(3,5);
 %! assert(t1 == 30 && t2 == 8)
 
 %!test
 %! % cell arrays specfies order, overriding symvar order
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(x*y, 12/y, 'vars', {y x});
 %! [t1, t2] = h(3, 6);
 %! assert(t1 == 18 && t2 == 4)
 %! h = function_handle(x*y, 12/y, 'vars', [y x]);
-%! warning(s)
 %! [t1, t2] = h(3, 6);
 %! assert(t1 == 18 && t2 == 4)
 
 %!test
 %! % cell arrays specfies order, overriding symvar order
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(x*y, 12/y, 'vars', {y x});
 %! [t1, t2] = h(3, 6);
 %! assert(t1 == 18 && t2 == 4)
 %! h = function_handle(x*y, 12/y, 'vars', [y x]);
-%! warning(s)
 %! [t1, t2] = h(3, 6);
 %! assert(t1 == 18 && t2 == 4)
 
-%!xtest
+%!test
 %! % Functions with different names in Sympy.
-%! % (will fail unless Sympy has Octave codegen)
 %! f = abs(x);  % becomes Abs(x)
 %! h = function_handle(f);
 %! assert(h(-10) == 10)
@@ -243,18 +252,16 @@ end
 
 %!test
 %! % 'file' with empty filename returns handle
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(2*x*y, 'file', '');
 %! assert(isa(h, 'function_handle'))
 %! assert(h(3,5)==30)
 %! h = function_handle(2*x*y, 'vars', {x y}, 'file', '');
-%! warning(s)
 %! assert(isa(h, 'function_handle'))
 %! assert(h(3,5)==30)
 
-%!xtest
+%!test
 %! % output to disk
-%! % (will fail unless Sympy has Octave codegen)
+%! fprintf('\n')
 %! f = function_handle(2*x*y, 2^x, 'vars', {x y z}, 'file', 'temp_test_output1');
 %! assert( isa(f, 'function_handle'))
 %! [a,b] = f(10,20,30);
@@ -263,9 +270,8 @@ end
 %! assert (b == 1024)
 %! delete('temp_test_output1.m')
 
-%!xtest
+%!test
 %! % output to disk: also works with .m specified
-%! % (will fail unless Sympy has Octave codegen)
 %! f = function_handle(2*x*y, 2^x, 'vars', {x y z}, 'file', 'temp_test_output2.m');
 %! assert( isa(f, 'function_handle'))
 %! [a,b] = f(10,20,30);
@@ -274,9 +280,8 @@ end
 %! assert (b == 1024)
 %! delete('temp_test_output2.m')
 
-%!xtest
+%!test
 %! % non-scalar outputs
-%! % (will fail unless Sympy has Octave codegen)
 %! H = [x y z];
 %! M = [x y; z 16];
 %! V = [x;y;z];
@@ -286,9 +291,8 @@ end
 %! assert(isequal(t2, [1 2; 3 16]))
 %! assert(isequal(t3, [1;2;3]))
 
-%!xtest
+%!test
 %! % non-scalar outputs in .m files
-%! % (will fail unless Sympy has Octave codegen)
 %! H = [x y z];
 %! M = [x y; z 16];
 %! V = [x;y;z];
