@@ -30,21 +30,26 @@
 %%
 %%
 %% @strong{Python executable} path/command:
+%%
 %% @example
 %% @group
-%% >> sympref python '/usr/bin/python'       @c doctest: +SKIP
-%% >> sympref python 'C:\Python\python.exe'  @c doctest: +SKIP
-%% >> sympref python 'N:\myprogs\py.exe'     @c doctest: +SKIP
+%% >> sympref python '/usr/bin/python'        % doctest: +SKIP
+%% >> sympref python 'C:\Python\python.exe'   % doctest: +SKIP
+%% >> sympref python 'N:\myprogs\py.exe'      % doctest: +SKIP
 %% @end group
 %% @end example
+%%
 %% Default is an empty string; in which case OctSymPy just runs
 %% @code{python} and assumes the path is set appropriately.
 %%
+%%
 %% @strong{Display} of syms:
+%%
 %% @example
 %% @group
 %% >> sympref display
 %%    @result{} ans = unicode
+%%
 %% @end group
 %% @group
 %% >> syms x
@@ -69,15 +74,21 @@
 %% >> sympref display default
 %% @end group
 %% @end example
+%%
 %% By default OctSymPy uses the unicode pretty printer to display
 %% symbolic expressions.  If that doesn't work (e.g., if you
 %% see @code{?} characters) then try the @code{ascii} option.
 %%
+%%
 %% @strong{Communication mechanism}:
+%%
 %% @example
+%% @group
 %% >> sympref ipc
 %%    @result{} ans = default
+%% @end group
 %% @end example
+%%
 %% The default will typically be the @code{popen2} mechanism which
 %% uses a pipe to communicate with Python and should be fairly fast.
 %% If that doesn't work, try @code{sympref display system} which is
@@ -101,19 +112,24 @@
 %% [for debugging, may not be supported long-term].
 %% @end itemize
 %%
+%%
 %% @strong{Reset}: reset the SymPy communication mechanism.  This can be
 %% useful after an error occurs and the connection with Python
 %% becomes confused.
-%% @example
-%% >> sympref reset     @c doctest: +SKIP
-%% @end example
 %%
-%% @strong{Snippets}: when displaying a sym object, we quote the SymPy
-%% representation (or a small part of it):
 %% @example
 %% @group
-%% >> syms x
-%% >> y = [pi x];
+%% >> sympref reset    % doctest: +SKIP
+%% @end group
+%% @end example
+%%
+%%
+%% @strong{Snippets}: when displaying a sym object, we can optionally
+%% quote a small part of the SymPy representation:
+%%
+%% @example
+%% @group
+%% >> syms x;  y = [pi x];
 %% >> sympref snippet on
 %% >> y
 %%    @result{} y = (sym 1×2 matrix)       “...([[pi, Symbol('x')]])”
@@ -125,8 +141,10 @@
 %% @end group
 %% @end example
 %%
+%%
 %% @strong{Default precision}: control the number of digits used by
 %% variable-precision arithmetic (see also the @ref{digits} command).
+%%
 %% @example
 %% @group
 %% >> sympref digits          % get
@@ -136,12 +154,22 @@
 %% @end group
 %% @end example
 %%
+%% Be @strong{quiet} by minimizing startup and diagnostics messages:
+%% @example
+%% @group
+%% >> sympref quiet
+%%    @result{} ans = 0
+%% >> sympref quiet on
+%% >> sympref quiet default
+%% @end group
+%% @end example
 %%
 %% Report the @strong{version} number:
+%%
 %% @example
 %% @group
 %% >> sympref version
-%%    @result{} 2.2.3
+%%    @result{} 2.3.0
 %% @end group
 %% @end example
 %%
@@ -171,10 +199,11 @@ function varargout = sympref(cmd, arg)
       sympref ('display', 'default')
       sympref ('digits', 'default')
       sympref ('snippet', 'default')
+      sympref ('quiet', 'default')
 
     case 'version'
       assert (nargin == 1)
-      varargout{1} = '2.2.3';
+      varargout{1} = '2.3.0';
 
     case 'display'
       if (nargin == 1)
@@ -213,11 +242,22 @@ function varargout = sympref(cmd, arg)
       if (nargin == 1)
         varargout{1} = settings.snippet;
       else
-	if (strcmpi(arg, 'default'))
-	  settings.snippet = false;  % Should be false for a release
-	else
+        if (strcmpi(arg, 'default'))
+          settings.snippet = false;  % Should be false for a release
+        else
           settings.snippet = tf_from_input(arg);
-	end
+        end
+      end
+
+    case 'quiet'
+      if (nargin == 1)
+        varargout{1} = settings.quiet;
+      else
+        if (strcmpi(arg, 'default'))
+          settings.quiet = false;
+        else
+          settings.quiet = tf_from_input(arg);
+        end
       end
 
     case 'python'
@@ -235,26 +275,34 @@ function varargout = sympref(cmd, arg)
       if (nargin == 1)
         varargout{1} = settings.ipc;
       else
+        verbose = ~sympref('quiet');
         sympref('reset')
         settings.ipc = arg;
         switch arg
           case 'default'
-            disp('Choosing the default [autodetect] octsympy communication mechanism')
+            msg = 'Choosing the default [autodetect] octsympy communication mechanism';
           case 'system'
-            disp('Forcing the system() octsympy communication mechanism')
+            msg = 'Forcing the system() octsympy communication mechanism';
           case 'popen2'
-            disp('Forcing the popen2() octsympy communication mechanism')
+            msg = 'Forcing the popen2() octsympy communication mechanism';
           case 'systmpfile'
-            disp('Forcing systmpfile ipc: warning: this is for debugging')
+            msg = 'Forcing systmpfile ipc: warning: this is for debugging';
           case 'sysoneline'
-            disp('Forcing sysoneline ipc: warning: this is for debugging')
+            msg = 'Forcing sysoneline ipc: warning: this is for debugging';
           otherwise
+            msg = '';
             warning('Unsupported IPC mechanism: hope you know what you''re doing')
         end
+        if (verbose)
+          disp(msg)
+        end
       end
 
     case 'reset'
-      disp('Resetting the octsympy communication mechanism');
+      verbose = ~sympref('quiet');
+      if (verbose)
+        disp('Resetting the octsympy communication mechanism');
+      end
       r = python_ipc_driver('reset', []);
 
       if (nargout == 0)
@@ -303,31 +351,62 @@ function r = tf_from_input(s)
 end
 
 
+%!test
+%! % test quiet, side effect of making following tests a bit less noisy!
+%! sympref quiet on
+%! a = sympref('quiet');
+%! assert(a == 1)
+
+%% Test for correct line numbers in Python exceptions.  These first tests
+%% happen with the default ipc---usually popen2.  We don't explicitly test
+%% popen2 because test suite should be portable.
+%!error <line 1> python_cmd('raise ValueError');
+%!error <line 1> python_cmd('raise ValueError', sym('x'));
+%!error <line 1> python_cmd('raise ValueError', sym([1 2 3; 4 5 6]));
+%!error <line 1> python_cmd('raise ValueError', {1; 1; 1});
+%!error <line 1> python_cmd('raise ValueError', struct('a', 1, 'b', 'word'));
+%!error <line 2> python_cmd( {'x = 1' 'raise ValueError'} );
+%!error <line 3> python_cmd( {'x = 1' 'pass' '1/0'} );
+%!error <line 3> python_cmd( {'a=1' 'b=1' 'raise ValueError' 'c=1' 'd=1'} );
+
+
 %!test
 %! % system should work on all system, but just runs sysoneline on windows
 %! fprintf('\nRunning some tests that reset the IPC and produce output\n');
 %! sympref('ipc', 'system');
-%! pause(1);
 %! syms x
-%! pause(2);
+
+%!error <line 1> python_cmd('raise ValueError')
+%!error <line 1> python_cmd('raise ValueError', sym('x'))
+%!error <line 1> python_cmd('raise ValueError', struct('a', 1, 'b', 'word'))
+
 
 %!test
 %! % sysoneline should work on all systems
-%! fprintf('\n');
 %! sympref('ipc', 'sysoneline');
-%! pause(1);
 %! syms x
-%! pause(2);
+
+%!error <line 1> python_cmd('raise ValueError')
+%!error <line 1> python_cmd('raise ValueError', struct('a', 1, 'b', 'word'))
+
+
+%!test
+%! sympref('ipc', 'systmpfile');
+%! syms x
+
+%!error <line 1> python_cmd('raise ValueError')
+%!error <line 1> python_cmd('raise ValueError', struct('a', 1, 'b', 'word'))
+
 
 %!test
 %! sympref('defaults')
 %! assert(strcmp(sympref('ipc'), 'default'))
+%! sympref('quiet', 'on')
 
 %!test
-%! fprintf('\n');
 %! syms x
 %! r = sympref('reset');
-%! pause(1);
 %! syms x
-%! pause(2);
 %! assert(r)
+%! % ok, can be noisy again
+%! sympref('quiet', 'default')
